# FILE: PlotMulti.gp
# COPYRIGHT (c): Symmetric Research, 2011-2016
#
# This GnuPlot file plots data from two USBxCH boards twice.  The first
# plot uses sample number for the X axis variable, while the second uses
# seconds since 1970.  
#
# To run, start GnuPlot and use ...   load 'PlotMulti.gp'
#
# The exact same input signal was recorded on both boards.  From this pair 
# of plots it is clear that the boards started recording at slightly 
# different times.  So aligning the data based on sample number doesn't
# work well.  
#
# But when the data are aligned based on the time of each sample as 
# determined by the GPS timestamps included with the data for each board, 
# the alignment is excellent.  
#
# Small deviations in peak amplitudes are to be expected since the samples 
# on the two boards are taken at slightly different times and the amplitude 
# is changing during that time.
#



# REMOVE ANY PREVIOUS DEFAULT SETTINGS
#
#

reset




# COMMENT IN THE LINES BELOW FOR POSTSCRIPT FILE OUTPUT:
#
# Process the Postscript output from GnuPlot through Adobe Distiller
# to make a PDF.
#
# The GnuPlot set terminal postscript defaults are:
#
#    orientation = landscape
#         x size = 10" horizontal
#         y size =  7" vertical
#
#
#

#set terminal postscript
#set output 'PlotMulti.ps'



# DEFINE SOME HANDY CONSTANTS RELATED TO THE DATA FILE
#
#

DataFile0 = "Scope0/Scope0.fw8"
DataFile1 = "Scope1/Scope1.fw4"

StartTime  = 1466711549
StartPoint = 200


# Board 0, USB8CH columns

Col0Sample = 1                   # sample number    is in column 1
Col0Time   = 19                  # time in seconds  is in column 19
Col0Data   = 2                   # channel 0 data   is in column 2

# Board 1, USB4CH columns

Col1Sample = 1                   # sample number    is in column 1
Col1Time   = 15                  # time in seconds  is in column 15
Col1Data   = 2                   # channel 1 data   is in column 2





# DEFINE LABEL FOR AXES, PLOT AND KEY AND CONTROL RANGE OF AXES
#
# Offset on ylabel moves it 5 characters to the left (-5 in x direction,
# and 0 in the y direction) so it doesn't run into the count values.

set xlabel "Time in Seconds"           font "Helvetica Bold,14"
set ylabel "Data Counts"               font "Helvetica Bold,14"  offset -5,0
set title  "USBxCH Multi-Board Data"   font "Helvetica Bold,18"

KeyTitle0 = "Board 0, USB8CH data"
KeyTitle1 = "Board 1, USB4CH data"

set xtics nomirror                      font "Helvetica Bold,14"
set ytics nomirror                      font "Helvetica Bold,14"




# SET THE DATA PLOTTING STYLE
#
# The first set style says plot data with points connected by lines.  The
# next set styles defines linestyle 1 and 2 that can be referenced later by
# by the plot command.  The first style has black lines connecting small solid
# circle points, while the second has red lines connecting large open diamond
# points.
#

set style data linespoint
set style line 1 linetype -1 linewidth 1 pointtype 7 pointsize 1
set style line 2 linetype  1 linewidth 1 pointtype 12 pointsize 2



# NOW DO THE PLOTS
#
# Put two stacked vertically on one page
#


set multiplot layout 2,1

set xrange [StartPoint+520:StartPoint+600]
set yrange [-200000:200000]
set xlabel "Sample"
set title  "USBxCH Multi-Board Data vs Sample Number"

plot DataFile0 using Col0Sample:Col0Data linestyle 1 title KeyTitle0, \
     DataFile1 using Col1Sample:Col1Data linestyle 2 title KeyTitle1


set xrange [5.2:6]
set xlabel "Time in Seconds"
set title  "USBxCH Multi-Board Data vs GPS Time"

plot DataFile0 using ($19-StartTime):Col0Data linestyle 1 title KeyTitle0, \
     DataFile1 using ($15-StartTime):Col1Data linestyle 2 title KeyTitle1


pause -1 "Click either button to exit GnuPlot"

unset multiplot



# CLEAN UP SETTINGS
#
#

reset
