// FILE: Filt3.c
// COPYRIGHT: (c), Symmetric Research, 2018
//
// Source code for the Filt3.exe custom USBxCH BIN pipeline utility in C.
//
// Open the input and output pipes and process USBxCH BIN records as they
// arrive, filtering data on the specified analog channels using a three term
// filter described by the roots of the characteristic equation and two
// previous filter values as initial conditions.
//
// The three term filter equation is:
//
//        y(n) = b * y(n-1) + c * y(n-2) + scale * x(n)
//
// where b and c are coefficients applied to previous results and initial
// conditions y(0) and y(1) must be specified in order to begin filtering.
//
// The filtered results are passed to the output pipe.  Include the BinWrite or
// BinTee programs before Filt3 in the pipeline chain if you want to keep a copy
// of the unfiltered data too.
//
// In the code below, all filter parameters are DEFINED constants.  If you
// want to change them, assign new values and recompile.  To take filter
// parameters from the command line or a parameter file, modify the code.  See
// \SR\USBxCH\Exe\Filter.exe for a utility that already has those features.
//
// The file CustomPipeLib.c must also be compiled and linked in to provide the
// pipeline functions used here.
//
//

// Include files:

#include <stdio.h>
#include <stdlib.h>

#include "CustomPipeLib.h"  // << for SR prototypes, BIN, and other defines



// Filter parameter defines:
//
// LAMBDA1, LAMBDA2 = roots of the characteristic equation L**2 - b*L + c = 0
// Y0,Y1            = initial conditions as filter value for 2 previous terms
// APPLY_CHANNEL#   = 1 to filter channel #, 0 to leave alone
//
// The lambda roots must be less than 1 for the filter to be stable.
//
// All channels with the "APPLY_CHANNEL" define set to 1 will be filtered.
// If you have channel you'd like to leave alone, set its APPLY define to 0.

//
// Tune these parameters as appropriate for your application ...
//
//

#define LAMBDA1                 0.9     // ** MUST ** be < 1.0
#define LAMBDA2                 0.9     // ** MUST ** be < 1.0
#define Y0                      0.0
#define Y1                      0.0

#define APPLY_CHANNEL0          1
#define APPLY_CHANNEL1          0
#define APPLY_CHANNEL2          0
#define APPLY_CHANNEL3          0
#define APPLY_CHANNEL4          0
#define APPLY_CHANNEL5          0
#define APPLY_CHANNEL6          0
#define APPLY_CHANNEL7          0




// Filt3.c global variables:

double CoeffB;  // weight on first previous term
double CoeffC;  // weight on second previous term
double Scale;   // weight on current imput

int  ChannelApply[MAXANALOGCHANNELS] = {

        APPLY_CHANNEL0,  // list of channels to check
        APPLY_CHANNEL1,
        APPLY_CHANNEL2,
        APPLY_CHANNEL3,
        APPLY_CHANNEL4,
        APPLY_CHANNEL5,
        APPLY_CHANNEL6,
        APPLY_CHANNEL7,

        };

double ChannelNm0[MAXANALOGCHANNELS];    // current filtered point
double ChannelNm1[MAXANALOGCHANNELS];    // first previous point
double ChannelNm2[MAXANALOGCHANNELS];    // second previous point


// Prototypes:

void InitCoeffAndPrev();
void ReportFilterParameters( void );
void ApplyFilter( BIN *pPt );




// Main program:

int main( int argc, char *argv[] ) {

        char         *NextProg;
        char         InputPipeName[MAXSTR];
        char         OutputPipeName[MAXSTR];
        SRPIPEHANDLE hInputPipe;
        SRPIPEHANDLE hOutputPipe;
        BIN          Pt;



        // Open the report file and announce the program.

        CustomReportOpen( "Filt3" );
        CustomReportPrint( "Filt3 starting\n" );



        // Process the command line for the "next program" name.

        if ( argc == 2 )

                NextProg = argv[1];

        else { // it is a bad command line, quit ...

                CustomReportPrint( "Usage: Filt3 NextProg\n" );
                exit( 0 );
                }



        // Form the input and output pipe names.

        sprintf( InputPipeName,  "%s%s", REQUIRED_PIPE_PREFIX, "Filt3" );
        sprintf( OutputPipeName, "%s%s", REQUIRED_PIPE_PREFIX, NextProg );



        // Open the input and output pipes.

        hInputPipe  = CustomOpenInputPipe( InputPipeName );
        hOutputPipe = CustomOpenOutputPipe( OutputPipeName );

          // >> careful programmers might add error checking to confirm the
          // >> input and output pipes were successfully opened ...



        // Initialize the computed values of coefficients and scale
        // and the per channel previous filter term values.

        InitCoeffAndPrev();



        // Report the filter parameters.

        ReportFilterParameters();




        // Loop reading from the input pipe.

        while ( CustomReadInputPipe( hInputPipe, &Pt ) ) {


                // An input pipe sample point has been returned ...
                //
                // ... filter it ...

                ApplyFilter( &Pt );



                // Pass the filtered BIN sample point on to output pipe ...

                CustomWriteOutputPipe( hOutputPipe, &Pt );


                // ... and continue on waiting for the input pipe ...

                }



        // The input pipe has failed ( upstream is probably closing down ),
        // close all gracefully ...

        CustomCloseInputPipe(  hInputPipe );
        CustomCloseOutputPipe( hOutputPipe );

        exit( 0 );

}


//------------------------------------------------------------------------------
// ROUTINE: InitCoeffAndPrev
// PURPOSE: Initialize global weighting parameters and the first two previous
//          filter terms for each channel.
//
//------------------------------------------------------------------------------
void InitCoeffAndPrev( void ) {

        int i;


        // Init global weighting parameters.

        CoeffB = LAMBDA1 + LAMBDA2;             // weight on first previous term
        CoeffC = -1 * LAMBDA1 * LAMBDA2;        // weight on second previous term
        Scale  = (1 - LAMBDA1) * (1 - LAMBDA2); // weight on current input


        // Init first two previous filter terms.

        for ( i = 0 ; i < MAXANALOGCHANNELS ; i++ ) {
                ChannelNm1[i] = Y1;            // first previous point y(n-1)
                ChannelNm2[i] = Y0;            // second previous point y(n-2)
                }
}


//------------------------------------------------------------------------------
// ROUTINE: ReportFilterParameters
//
// PURPOSE: Report the filter parameters ...
//
//------------------------------------------------------------------------------
void ReportFilterParameters( void ) {

        int  i;
        char MsgBuffer[MAXSTR];


        // Report defined parameter values.

        CustomReportPrint( "Defined parameters:\n" );
        sprintf( MsgBuffer, "Lambda1 = %lf\n", LAMBDA1 ); CustomReportPrint( MsgBuffer );
        sprintf( MsgBuffer, "Lambda2 = %lf\n", LAMBDA2 ); CustomReportPrint( MsgBuffer );
        sprintf( MsgBuffer, "     Y0 = %lf\n", Y0 );      CustomReportPrint( MsgBuffer );
        sprintf( MsgBuffer, "     Y1 = %lf\n", Y1 );      CustomReportPrint( MsgBuffer );

        for ( i = 0 ; i < MAXANALOGCHANNELS ; i++ ) {

                sprintf( MsgBuffer, "Apply Channel %d = %d\n", i, ChannelApply[i] );
                CustomReportPrint( MsgBuffer );

                }


        // Report computed parameter values.
        
        CustomReportPrint( "\nComputed parameters:\n" );
        sprintf( MsgBuffer, "CoeffB = %lf\n", CoeffB ); CustomReportPrint( MsgBuffer );
        sprintf( MsgBuffer, "CoeffC = %lf\n", CoeffC ); CustomReportPrint( MsgBuffer );
        sprintf( MsgBuffer, " Scale = %lf\n", Scale ); CustomReportPrint( MsgBuffer );
}


//------------------------------------------------------------------------------
// ROUTINE: ApplyFilter
//
// PURPOSE: Apply the recursive filter formula to the next point for all
//          requested channels.
//
//------------------------------------------------------------------------------
void ApplyFilter( BIN *pPt ) {

        int i;


        // Compute next filter point for each channel.

        for ( i = 0 ; i < MAXANALOGCHANNELS ; i++ ) {

                if ( ChannelApply[i] == 1 ) { // filter requested

                        // Compute this filter point.

                        ChannelNm0[i]  = CoeffB * ChannelNm1[i] +
                                         CoeffC * ChannelNm2[i] +
                                         Scale  * pPt->Analog[i];

                        pPt->Analog[i] = (int)ChannelNm0[i]; // save rounded version


                        // Update two previous filter terms.
                        
                        ChannelNm2[i] = ChannelNm1[i];
                        ChannelNm1[i] = ChannelNm0[i]; // pass on full version
                        }

                // else pPt->Analog[i] is unchanged

                }  // end for i < < MAXANALOGCHANNELS
}
