// FILE: TriggerOnD0.c
// COPYRIGHT: (c), Symmetric Research, 2018
//
// Source code for the TriggerOnD0.exe custom USBxCH BIN pipeline utility in C.
//
// This utility opens the input and output pipes, and then processes USBxCH BIN
// records as they arrive, checking for a trigger signal on the USBxCH digital
// input bit D0.
//
// A trigger is signaled when the voltage on D0 ( = DB25 pin 1 ) is 3 volts.
// Normally D0 should idle at 0 volts.  The trigger must be maintained for at
// least one analog sampling period.  After that NTRIGGER BIN sample points
// are marked as triggered.
//
// The trigger signal is recorded in the BIN.Trigger record field.  This field
// allows downstream pipeline utilities to know about the trigger.
//
// Examples of downstream processing might be:
//
//      ScopeDisplay can view the trigger signal as a trace
//      Bin2Asc can be set to only process BIN records marked "triggered"
//
// This is a simple piece of code that should be modified as appropriate for
// specific applications.  Many extensions are possible ...
//
//
// The file CustomPipeLib.c must be compiled and linked with this code to
// provide the pipeline functions used here.
//
//
//

// Include files:

#include <stdio.h>
#include <stdlib.h>

#include "CustomPipeLib.h"      // for prototypes, BIN, and other SR defines




// Trigger.exe DEFINES:


// NTRIGGER: POST TRIGGER "trailer" LENGTH ...
//
// This sets the number of sample points to mark as "triggered" after
// the D0 0 to 1 signal has appeared.
//
// Change this "trailer" length as appropriate.
//

#define NTRIGGER        1000



// Report file name and messages:
//
//
#define ReportFileName  "TriggerOnD0"

#define ReportMsg1    "TriggerOnD0 starting\n"
#define ReportMsg2    "Usage: TriggerOnD0 NextProg\n"
#define ReportMsg3    "Trigger found and saving NTRIGGER points\n"
#define ReportMsg4    "Trigger finished\n"






// Main program:

int main( int argc, char *argv[] ) {

        char         *NextProg;
        char         InputPipeName[MAXSTR];
        char         OutputPipeName[MAXSTR];
        SRPIPEHANDLE hInputPipe;
        SRPIPEHANDLE hOutputPipe;
        BIN          Pt;

        static int   Triggered = 0;
        static int   npoints = 0;



        // Open the report file and announce the program.

        CustomReportOpen( ReportFileName );
        CustomReportPrint( ReportMsg1 );



        // Process the command line for the "next program" name.

        if ( argc == 2 )

                NextProg = argv[1];

        else {
                // it is a bad command line, quit ...

                CustomReportPrint( ReportMsg2 );
                exit( 0 );
                }



        // Form the input and output pipe names.

        sprintf( InputPipeName,  "%s%s", REQUIRED_PIPE_PREFIX, "TriggerOnD0" );
        sprintf( OutputPipeName, "%s%s", REQUIRED_PIPE_PREFIX, NextProg );



        // Open the input and output pipes.

        hInputPipe  = CustomOpenInputPipe( InputPipeName );
        hOutputPipe = CustomOpenOutputPipe( OutputPipeName );

          // >> careful programmers might add error checking to confirm the
          // >> input and output pipes were successfully opened ...



        // Loop reading from the input pipe.

        while ( CustomReadInputPipe( hInputPipe, &Pt ) ) {

                // Ok ... a sample point BIN record has been returned by
                //     CustomReadInputPipe() at &Pt ...


                // Check for a D0 trigger signal:
                //
                //   this assumes D0 idles at 0,
                //     when D0 goes to 1 then a trigger is asserted ...
                //

                if ( Pt.DigitalAll & 0x1 )  // <<< mask off and test D0

                        Triggered = 0xFF;


                // Once triggered, mark the next NTRIGGER BIN records:
                //
                // In this simple code, no futher events will be marked after
                // NTRIGGER points have passed.  Change this behavior if needed
                // depending on the nature of your application.
                //

                if ( Triggered && ( npoints < NTRIGGER ) ) {

                                Pt.Trigger = 0xFF;
                                npoints++;

                                if ( npoints == 1 )
                                     CustomReportPrint( ReportMsg3 );

                                if ( npoints == NTRIGGER )
                                     CustomReportPrint( ReportMsg4 );

                                }



                // Pass the BIN sample point on to the output pipe:

                CustomWriteOutputPipe( hOutputPipe, &Pt );

                }



        // If the input pipe has failed, upstream is probably closing down,
        // close all gracefully ...

        CustomCloseInputPipe(  hInputPipe );
        CustomCloseOutputPipe( hOutputPipe );


        exit( 0 );

}

