' FILE: VbCustomPipeLib.vb
' COPYRIGHT: (c), Symmetric Research, 2015-2017
'
' Source code for accessing USBxCH BIN data from a SrPipe in Visual Basic.
'
' For this code to work, VbSrUsbXchBin.vb MUST be included in the project.
'

Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Threading



Public Module VbCustomPipeLib

    ' Constants to share with VbAbs

    Public Const REQUIRED_PIPE_PREFIX As String = "\\.\pipe\SrPipe"


    ' Constants for SR and standard kernel32.dll functions:

    Private Const PIPE_ACCESS_OUTBOUND As Integer = &H2
    Private Const FILE_FLAG_FIRST_PIPE_INSTANCE As Integer = &H80000
    Private Const PIPE_TYPE_BYTE As Integer = &H0
    Private Const PIPE_READMODE_BYTE As Integer = &H0
    Private Const PIPE_WAIT As Integer = &H0
    Private Const GENERIC_READ As Integer = &H80000000
    Private Const FILE_WRITE_ATTRIBUTES As Integer = &H100
    Private Const FILE_SHARE_READ As Integer = &H1
    Private Const FILE_SHARE_WRITE As Integer = &H2
    Private Const FILE_ATTRIBUTE_NORMAL As Short = &H80S
    Private Const OPEN_EXISTING As Short = 3
    Private Const ERROR_SUCCESS As Integer = 0
    Private Const ERROR_BAD_PIPE As Integer = 230
    Private Const ERROR_PIPE_BUSY As Integer = 231
    Private Const ERROR_NO_DATA As Integer = 232
    Private Const ERROR_PIPE_NOT_CONNECTED As Integer = 233
    Private Const ERROR_PIPE_CONNECTED As Integer = 535
    Private Const SR_PIPE_TOTAL_WAIT As Integer = 4000 ' 4 seconds
    Private Const SR_PIPE_SIZE As Integer = NSIZE_BINDATA * 10000 * 2 ' sizeof(BIN) * max sample rate * 2 seconds

    ' Global (to this library) variables:
    
    Private Dim RptFp As StreamWriter


    ' Function declarations for standard kernel32.dll functions:

    Private Declare Function CreateNamedPipe Lib "kernel32" Alias "CreateNamedPipeA" _
              (ByVal lpFileName As String, _
              ByVal dwOpenMode As Integer, _
              ByVal dwPipeMode As Integer, _
              ByVal dwMaxInstances As Integer, _
              ByVal dwOutBufferSize As Integer, _
              ByVal dwInBufferSize As Integer, _
              ByVal dwDefaultTimeOut As Integer, _
              ByVal lpSecurityAttributes As IntPtr) _
              As Integer

    Private Declare Function PeekNamedPipe Lib "kernel32" Alias "PeekNamedPipe" _
             (ByVal hPipeHandle As Integer, _
              ByRef lpBuffer As BinLayout, _
              ByVal nBufferSize As Integer, _
              ByRef lpBytesRead As Integer, _
              ByRef lpTotalBytesAvail As Integer, _
              ByRef lpBytesLeftThisMessage As Integer) _
              As Integer

    Private Declare Function ConnectNamedPipe Lib "kernel32" Alias "ConnectNamedPipe" _
              (ByVal hPipeHandle As Integer, _
               ByVal lpOverlapped As IntPtr) _
              As Integer

    Private Declare Function DisconnectNamedPipe Lib "kernel32" Alias "DisconnectNamedPipe" _
              (ByVal hPipeHandle As Integer) _
              As Integer

    Private Declare Function FlushPipeBuffers Lib "kernel32" Alias "FlushFileBuffers" _
              (ByVal hPipeHandle As Integer) _
              As Integer

    Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" _
              (ByVal lpFileName As String, _
              ByVal dwDesiredAccess As Integer, _
              ByVal dwShareMode As Integer, _
              ByVal lpSecurityAttributes As IntPtr, _
              ByVal dwCreationDisposition As Integer, _
              ByVal dwFlagsAndAttributes As Integer, _
              ByVal hTemplateFile As IntPtr) _
              As Integer

    Private Declare Function ReadFile Lib "kernel32" _
              (ByVal hFile As Integer, _
              ByRef Buffer As BinLayout, _
              ByVal nNumberOfBytesToRead As Integer, _
              ByRef nNumberOfBytesRead As Integer, _
              ByVal lpOverlapped As IntPtr) _
              As Integer

    Private Declare Function WriteFile Lib "kernel32" _
              (ByVal hFile As Integer, _
              ByRef Buffer As BinLayout, _
              ByVal nNumberOfBytesToWrite As Integer, _
              ByRef nNumberOfBytesWritten As Integer, _
              ByVal lpOverlapped As IntPtr) _
              As Integer

    Private Declare Function CloseHandle Lib "kernel32" _
          (ByVal hObject As Integer) _
          As Integer

    

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomReportOpen
    ' PURPOSE: Use open a report file with a compatible name.
    '---------------------------------------------------------------------------
    Public Sub VbCustomReportOpen( ByVal UtilityName As String )

        Dim RptName As String

        RptName = "SrRpt-" & UtilityName & ".rpt"

        RptFp = New StreamWriter(RptName, True) ' Create
        

    End Sub

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomReportPrint
    ' PURPOSE: Write provided message to the open report file.
    '---------------------------------------------------------------------------
    Public Sub VbCustomReportPrint( ByVal Msg As String )

        RptFp.Write( Msg )
        RptFp.Flush

    End Sub

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomErrorExit
    ' PURPOSE: Write provided message to the report file, close it, and exit.
    '---------------------------------------------------------------------------
    Public Sub VbCustomErrorExit( ByVal Msg As String )

        RptFp.Write( Msg )
        RptFp.Close

        Environment.Exit( 1 )

    End Sub

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomPipeError
    ' PURPOSE: Translate provided error code into a reason for failure and
    '          output to the report.
    '---------------------------------------------------------------------------
    Public Sub VbCustomPipeError( ByVal errCode As Integer, ByVal Caller As String )

        Dim Reason As String

        If errCode = ERROR_BAD_PIPE Then
            Reason = " failed on pipe because state is invalid, err= "
        Else If errCode = ERROR_PIPE_BUSY Then
            Reason = " failed on pipe because pipe is busy, err= "
        Else If errCode = ERROR_NO_DATA Then
            Reason = " failed on pipe because pipe is closed, err= "
        Else If errCode = ERROR_PIPE_NOT_CONNECTED Then
            Reason = " failed on pipe because other end is closed, err= "
        Else
            Reason = " failed on pipe because of unknown error, err= "
        End If
        VbCustomReportPrint( Caller & Reason & errCode & vbCrLf )

    End Sub
        

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomOpenInputPipe
    ' PURPOSE: Use the CreateFile function to open an existing named pipe for
    '          input.  Returns a handle to the pipe.
    '---------------------------------------------------------------------------
    Public Function VbCustomOpenInputPipe( ByVal InputPipeName As String ) _
                                           As Integer

            Dim hInputPipe As Integer
            Dim AccessMode As Integer
            Dim ShareMode As Integer
            Dim MsgBuffer As String

            ' Initialize mode variables
            
            AccessMode = (GENERIC_READ Or FILE_WRITE_ATTRIBUTES)
            ShareMode  = (FILE_SHARE_READ Or FILE_SHARE_WRITE)
            
            While 1

                ' Open the named pipe for binary reading.

                hInputPipe = CreateFile( InputPipeName, _
                                         AccessMode, _
                                         ShareMode, _
                                         IntPtr.Zero, _
                                         OPEN_EXISTING, _
                                         FILE_ATTRIBUTE_NORMAL, _
                                         IntPtr.Zero )

                ' If successful, leave infinite loop.

                If hInputPipe <> INVALID_HANDLE_VALUE Then
                    Exit While
                End If

                ' Otherwise, wait and try again.

                VbCustomSleep( 500 )

            End While

            MsgBuffer = "Opened input pipe " & InputPipeName & vbCrLf
            VbCustomReportPrint( MsgBuffer )

            VbCustomOpenInputPipe = hInputPipe

    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomPeekInputPipe
    ' PURPOSE: Use the PeekNamedPipe function see if one BIN complete data point
    '          is available on the opened named pipe.  Returns 0 on error.
    '          Also fills in the Ready parameter, 1 for ready, 0 otherwise.
    '---------------------------------------------------------------------------
    Public Function VbCustomPeekInputPipe( ByVal hInputPipe As Integer, _
                                           ByRef Ready As Integer ) _
                                           As Integer
        Dim errCode As Integer
        Dim numAvailable As Integer
        Dim Pt As BinLayout


        ' Initialize variables.
        
        numAvailable = 0
        Ready = 0


        ' Peek to see if one entire Pt is available.

        PeekNamedPipe( hInputPipe, _
                       Pt, _
                       NSIZE_BINDATA, _
                       IntPtr.Zero, _
                       numAvailable, _
                       IntPtr.Zero )

        errCode = Err.LastDllError
        If errCode <> ERROR_SUCCESS Then
            VbCustomPipeError( errCode, "Peek" )
            Return 0
        End If


        ' Determine if there are enough bytes ready.
        
        If numAvailable < NSIZE_BINDATA
            Ready = 0
        Else
            Ready = 1
        End If
        
        Return 1
    
    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomReadInputPipe
    ' PURPOSE: Use the ReadFile function to read one BIN data point from an
    '          opened named pipe.  Returns 0 on error.  Also fills in the
    '          Pt parameters.
    '---------------------------------------------------------------------------
    Public Function VbCustomReadInputPipe( ByVal hInputPipe As Integer, _
                                           ByRef Pt As BinLayout ) _
                                           As Integer
        Dim errCode As Integer
        Dim numAvailable As Integer
        Dim bytesRead As Integer

        ' Loop around until one entire Pt is available

        numAvailable = 0

        While numAvailable < NSIZE_BINDATA

            PeekNamedPipe( hInputPipe, _
                           Pt, _
                           NSIZE_BINDATA, _
                           IntPtr.Zero, _
                           numAvailable, _
                           IntPtr.Zero )

            errCode = Err.LastDllError
            If errCode <> ERROR_SUCCESS Then
                VbCustomPipeError( errCode, "Peek to read" )
                Return 0
            End If
            
        End While


        ' Read BinData from named pipe.

        VbCustomReadInputPipe = ReadFile(hInputPipe, _
                                         Pt, _
                                         NSIZE_BINDATA, _
                                         bytesRead, _
                                         IntPtr.Zero)

        errCode = Err.LastDllError
        If errCode <> ERROR_SUCCESS Then
            VbCustomPipeError( errCode, "Read" )
            Return 0
        End If


        ' Verify we've got a BIN structure by checking its ID.

        If Pt.Id = BIN_ID Then
            Return 1
        Else
            VbCustomReportPrint( "Invalid BIN structure read" & vbCrLf )
            Return 0
        End If
    
    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomCloseInputPipe
    ' PURPOSE: Use the CloseHandle function to close the client end of an opened
    '          named pipe.  Returns 0 on error.
    '---------------------------------------------------------------------------
    Public Function VbCustomCloseInputPipe( ByVal hInputPipe As Integer ) _
                                            As Integer
        ' Close the named pipe.

        VbCustomCloseInputPipe = CloseHandle( hInputPipe )
        VbCustomReportPrint( "Closed input pipe" & vbCrLf )

    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomOpenOutputPipe
    ' PURPOSE: Use the CreateFile function to open an existing named pipe for
    '          output.  Returns a handle to the pipe.
    '---------------------------------------------------------------------------
    Public Function VbCustomOpenOutputPipe( ByVal OutputPipeName As String ) _
                                            As Integer

        Dim hOutputPipe As Integer
        Dim PipeSize As Integer
        Dim OpenMode As Integer
        Dim PipeMode As Integer
        Dim NullPipeName As String
        Dim MsgBuffer As String
        Dim errCode As Integer
        Dim ok As Integer


        ' Check for NULL pipe, return invalid handle if so.

        NullPipeName = REQUIRED_PIPE_PREFIX & "NULL"
        
        If OutputPipeName = NullPipeName Then
            VbCustomReportPrint( "Null output pipe requested, so no open needed" & vbCrLf )
            Return INVALID_HANDLE_VALUE
        End If
        


        ' Initialize mode and size variables.

        OpenMode = (PIPE_ACCESS_OUTBOUND Or FILE_FLAG_FIRST_PIPE_INSTANCE)
        PipeMode = (PIPE_TYPE_BYTE Or PIPE_READMODE_BYTE Or PIPE_WAIT)
        PipeSize = SR_PIPE_SIZE


        ' Open the named pipe.

        hOutputPipe = CreateNamedPipe(OutputPipeName, _
                                      OpenMode, _
                                      PipeMode, _
                                      1, _
                                      PipeSize, _
                                      PipeSize, _
                                      SR_PIPE_TOTAL_WAIT, _
                                      IntPtr.Zero)

        If hOutputPipe = INVALID_HANDLE_VALUE Then
            VbCustomErrorExit( "Failed to open output pipe" )
        End If


        ' Wait until reader is connected before returning.

        ok = ConnectNamedPipe( hOutputPipe, IntPtr.Zero )
        
        errCode = Err.LastDllError

        If (ok = 0) And (errCode <> ERROR_PIPE_CONNECTED) Then
            VbCustomErrorExit( "Can't connect output pipe" )
        End If

        MsgBuffer = "Opened output pipe " & OutputPipeName & vbCrLf
        VbCustomReportPrint( MsgBuffer )

        VbCustomOpenOutputPipe = hOutputPipe

    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomWriteOutputPipe
    ' PURPOSE: Use the WriteFile function to write one BIN data point from an
    '          opened named pipe.  Returns 0 on error.  Also fills in the
    '          Pt parameter.
    '---------------------------------------------------------------------------
    Public Function VbCustomWriteOutputPipe( ByVal hOutputPipe As Integer, _
                                             ByRef Pt As BinLayout ) _
                                             As Integer

        Dim errCode As Integer
        Dim bytesWritten As Integer


        ' If handle for NULL pipe is found, don't write, just return success.

        If hOutputPipe = INVALID_HANDLE_VALUE Then
            Return 1
        End If


        ' Write BIN data to named pipe, this should block until all
        ' requested bytes have been written.

        VbCustomWriteOutputPipe = WriteFile(hOutputPipe, _
                                            Pt, _
                                            NSIZE_BINDATA, _
                                            bytesWritten, _
                                            IntPtr.Zero)
        errCode = Err.LastDllError
        If errCode <> ERROR_SUCCESS Then
            VbCustomPipeError( errCode, "Write" )
            Return 0
        End If
        
        Return 1

    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomCloseOutputPipe
    ' PURPOSE: Use the CloseHandle function to close the client end of an opened
    '          named pipe.  Returns 0 on error.
    '---------------------------------------------------------------------------
    Public Function VbCustomCloseOutputPipe( ByVal hOutputPipe As Integer ) _
                                             As Integer

        ' If handle for NULL pipe is found, don't close, just return success.

        If hOutputPipe = INVALID_HANDLE_VALUE Then
            VbCustomReportPrint( "Null output pipe requested, so no close needed" & vbCrLf )
            Return 1
        End If

        ' Close the named pipe.

        FlushPipeBuffers( hOutputPipe )    ' wait until client is finished reading
        DisconnectNamedPipe( hOutputPipe ) ' force close on client end
        VbCustomCloseOutputPipe = CloseHandle( hOutputPipe ) ' close on own end

        VbCustomReportPrint( "Closed output pipe" & vbCrLf )

    End Function

    '---------------------------------------------------------------------------
    ' ROUTINE: VbCustomSleep
    ' PURPOSE: Wait for ms milliseconds.
    '---------------------------------------------------------------------------
    Public Sub VbCustomSleep( ByVal ms As Integer )

        Thread.Sleep( ms )

    End Sub

End Module
